from cpl_core.application import StartupExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord import get_discord_collection
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum

from modules.admin.command.restart_command import RestartCommand
from modules.admin.command.shutdown_command import ShutdownCommand
from modules.base.command.afk_command import AFKCommand
from modules.base.command.help_command import HelpCommand
from modules.base.command.info_command import InfoCommand
from modules.base.command.ping_command import PingCommand
from modules.base.events.base_on_command_error_event import BaseOnCommandErrorEvent
from modules.moderator.command.purge_command import PurgeCommand
from modules.base.command.user_info_command import UserInfoCommand
from modules.base.events.base_on_member_join_event import BaseOnMemberJoinEvent
from modules.base.events.base_on_member_remove_event import BaseOnMemberRemoveEvent
from modules.base.events.base_on_message_event import BaseOnMessageEvent
from modules.base.events.base_on_voice_state_update_event import BaseOnVoiceStateUpdateEvent
from modules.boot_log.boot_log_on_ready_event import BootLogOnReadyEvent
from modules.database.database_on_ready_event import DatabaseOnReadyEvent
from modules.permission.events.permission_on_member_update_event import PermissionOnMemberUpdateEvent
from modules.permission.events.permission_on_ready_event import PermissionOnReadyEvent


class StartupDiscordExtension(StartupExtensionABC):

    def __init__(self):
        pass

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_discord()
        dc = get_discord_collection(services)
        """ commands """
        # admin
        dc.add_command(RestartCommand)
        dc.add_command(ShutdownCommand)
        # moderator
        dc.add_command(PurgeCommand)
        # simple
        dc.add_command(AFKCommand)
        dc.add_command(HelpCommand)
        dc.add_command(InfoCommand)
        dc.add_command(PingCommand)
        dc.add_command(UserInfoCommand)
        """ events """
        # on_command_error
        dc.add_event(DiscordEventTypesEnum.on_command_error.value, BaseOnCommandErrorEvent)
        # on_member_join
        dc.add_event(DiscordEventTypesEnum.on_member_join.value, BaseOnMemberJoinEvent)
        # on_member_remove
        dc.add_event(DiscordEventTypesEnum.on_member_join.value, BaseOnMemberRemoveEvent)
        # on_member_update
        dc.add_event(DiscordEventTypesEnum.on_member_update.value, PermissionOnMemberUpdateEvent)
        # on_message
        dc.add_event(DiscordEventTypesEnum.on_message.value, BaseOnMessageEvent)
        # on_voice_state_update
        dc.add_event(DiscordEventTypesEnum.on_voice_state_update.value, BaseOnVoiceStateUpdateEvent)
        # on_ready
        dc.add_event(DiscordEventTypesEnum.on_ready.value, DatabaseOnReadyEvent)
        dc.add_event(DiscordEventTypesEnum.on_ready.value, PermissionOnReadyEvent)
        dc.add_event(DiscordEventTypesEnum.on_ready.value, BootLogOnReadyEvent)  # has to be last
