from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC

from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC


class ClientUtilsService(ClientUtilsServiceABC):

    def __init__(
            self,
            config: ConfigurationABC,
            logger: LoggerABC,
            bot: DiscordBotServiceABC,
            servers: ServerRepositoryABC,
            clients: ClientRepositoryABC,
            message_service: MessageServiceABC,
            db: DatabaseContextABC,
            t: TranslatePipe
    ):
        ClientUtilsServiceABC.__init__(self)
        self._config = config
        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._clients = clients
        self._message_service = message_service
        self._db = db
        self._t = t

    def received_command(self, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.server_id)
        client.received_command_count += 1
        self._clients.update_client(client)
        self._db.save_changes()

    def moved_user(self, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.server_id)
        client.moved_users_count += 1
        self._clients.update_client(client)
        self._db.save_changes()

    def get_client(self, dc_ic: int, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.server_id)
        return client

    async def check_if_bot_is_ready_yet_and_respond(self, ctx: Context) -> bool:
        if self._config.get_configuration('IS_READY') == 'true':
            return True

        self._logger.debug(__name__, f'Bot is not ready yet {self._t.transform("common.errors.bot_not_ready_yet")}')
        await self._message_service.send_ctx_msg(ctx, self._t.transform('common.errors.bot_not_ready_yet'), without_tracking=True)
        return False
